@@ -20,32 +20,35 @@
 import os, copy, time
 import preludedb, CheetahFilters
 
-from prewikka import Config, Log, Database, IDMEFDatabase, ParametersNormalizer, \
-     User, Auth, DataSet, Error, utils, siteconfig
+import prewikka.views
+from prewikka import view, Config, Log, Database, IDMEFDatabase, \
+     User, Auth, DataSet, Error, utils, siteconfig, localization
 
 
-class InvalidQueryError(Error.SimpleError):
-    def __init__(self, query):
-        Error.SimpleError.__init__(self, "Query error", "Invalid query")
-
-
-
-class PermissionDeniedError(Error.SimpleError):
-    def __init__(self, user, action_name):
-        Error.SimpleError.__init__(self, "Permission Denied",
-                                   "User %s cannot access action %s" % (user, action_name))
-
+class InvalidQueryError(Error.PrewikkaUserError):
+    def __init__(self, message):
+        Error.PrewikkaUserError.__init__(self, "Invalid query", message, log=Log.ERROR)
 
 
+class Logout(view.View):
+    view_name = "logout"
+    view_parameters = view.Parameters
+    view_permissions = [ ]
+                
+    def render(self):
+        self.env.auth.logout(self.request)
+                    
+                    
 def init_dataset(dataset, config, request):
     interface = config.interface
     dataset["document.title"] = "[PREWIKKA]"
+    dataset["document.charset"] = localization.getCurrentCharset()
     dataset["document.css_files"] = [ "prewikka/css/style.css" ]
     dataset["document.js_files"] = [ "prewikka/js/functions.js" ]
     dataset["prewikka.title"] = interface.getOptionValue("title", "&nbsp;")
     dataset["prewikka.software"] = interface.getOptionValue("software", "&nbsp;")
     dataset["prewikka.place"] = interface.getOptionValue("place", "&nbsp;")
-    dataset["prewikka.date"] = time.strftime("%A %B %d %Y")
+    dataset["prewikka.date"] = localization.getDate()
 
     val = config.general.getOptionValue("external_link_new_window", "true")
     if (not val and config.general.has_key("external_link_new_window")) or (val == None or val.lower() in ["true", "yes"]):
@@ -58,9 +61,27 @@ def init_dataset(dataset, config, request):
         qstring = qstring[2:]
 
     dataset["prewikka.url.current"] = qstring
-    dataset["prewikka.url.referer"] = request.getReferer()
-
 
+    referer = request.getReferer()
+    idx = referer.rfind("/")
+    if idx != -1:
+        referer = referer[idx + 1:]
+        if referer[0:1] == "?":
+            referer = referer[1:]
+        
+    dataset["prewikka.url.referer"] = referer
+    
+    dataset["arguments"] = []
+    for name, value in request.arguments.items():
+        if name in ("_login", "_password"):
+            continue
+                
+        if name == "view" and value == "logout":
+            continue
+        
+        dataset["arguments"].append((name, value))
+            
+            
 def load_template(name, dataset):
     template = getattr(__import__("prewikka.templates." + name, globals(), locals(), [ name ]), name)(filtersLib=CheetahFilters)
         
@@ -105,6 +126,9 @@ class Core:
         class Env: pass
         self._env = Env()
         self._env.config = Config.Config(config)
+        self._env.max_aggregated_source = int(self._env.config.general.getOptionValue("max_aggregated_source", 10))
+        self._env.max_aggregated_target = int(self._env.config.general.getOptionValue("max_aggregated_target", 10))
+        self._env.default_locale = self._env.config.general.getOptionValue("default_locale", None)
         preludedb.preludedb_init()
         self._initDatabase()
         self._env.idmef_db = IDMEFDatabase.IDMEFDatabase(self._env.config.idmef_database)
@@ -130,27 +154,18 @@ class Core:
         
     def _initAuth(self):
         if self._env.auth.canLogout():
-            from prewikka import view
-
-            class Logout(view.View):
-                view_name = "logout"
-                view_parameters = view.Parameters
-                view_permissions = [ ]
-                
-                def render(self):
-                    self.env.auth.logout(self.request)
-            
             self._views.update(Logout().get())
 
     def _loadViews(self):
-        import prewikka.views
+        self._view_to_tab = { }
+        self._view_to_section = { }
         
-        self._views_position = { }
-        for section, tabs in (prewikka.views.events_section, prewikka.views.agents_section, prewikka.views.users_section,
-                              prewikka.views.about_section):
+        for section, tabs in (prewikka.views.events_section, prewikka.views.agents_section, 
+                              prewikka.views.settings_section, prewikka.views.about_section):
             for tab, views in tabs:
                 for view in views:
-                    self._views_position[view] = section, tabs, tab
+                    self._view_to_tab[view] = tab
+                    self._view_to_section[view] = section
                     
         self._views = { }
         for object in prewikka.views.objects:
@@ -181,7 +196,6 @@ class Core:
         object.user = user
         object.dataset = DataSet.DataSet()
         object.env = self._env
-        self._setupDataSet(object.dataset, request, user, view, parameters)
 
         return object
     
@@ -193,50 +207,60 @@ class Core:
         del view.env
         
     def _setupDataSet(self, dataset, request, user, view=None, parameters={}):
-        import prewikka.views
-
         init_dataset(dataset, self._env.config, request)
-        
-        if isinstance(self._env.auth, Auth.AnonymousAuth):
-            sections = prewikka.views.events_section, prewikka.views.agents_section, prewikka.views.about_section
-        else:
-            sections = prewikka.views.events_section, prewikka.views.agents_section, prewikka.views.users_section, \
-                       prewikka.views.about_section
+       
+        is_anon = isinstance(self._env.auth, Auth.AnonymousAuth)
+        sections = prewikka.views.events_section, prewikka.views.agents_section, prewikka.views.settings_section, \
+                   prewikka.views.about_section
 
+        section_to_tabs = { }
         dataset["interface.sections"] = [ ]
-        if user:
-            for section_name, tabs in sections:
-                viewable_tabs = 0
-                for tab_name, views in tabs:
-                    default_view = views[0]
-                    if user.has(self._views[default_view]["permissions"]):
-                        viewable_tabs += 1
-
-                if viewable_tabs > 0:
-                    dataset["interface.sections"].append((section_name,
-                                                          utils.create_link(tabs[0][1][0])))
-                    
-        import prewikka.view
-
-        if view and self._views_position.has_key(view["name"]):
-            active_section, tabs, active_tab = self._views_position[view["name"]]
-        elif isinstance(parameters, prewikka.view.RelativeViewParameters):
-            active_section, tabs, active_tab = self._views_position[parameters["origin"]]
-        else:
-            active_section, tabs, active_tab = "", "", ""
-
-        dataset["interface.active_section"] = active_section
-
-        dataset["interface.tabs"] = [ ]
-        if user:
-            for tab, views in tabs:
-                if user.has(self._views[views[0]]["permissions"]):
-                    dataset["interface.tabs"].append((tab, utils.create_link(views[0])))
+        for section_name, tabs in sections:
+            first_tab = None   
+   
+            for tab_name, views in tabs:
+                view_name = views[0]
         
-        dataset["interface.tabs"] = [ (tab, utils.create_link(views[0])) for tab, views in tabs ]
-        dataset["interface.active_tab"] = active_tab
+                if is_anon and tab_name == "Users":
+                    continue
+        
+                if not user or is_anon or user.has(self._views[view_name]["permissions"]):
+                    if not first_tab:
+                        first_tab = view_name
+                        section_to_tabs[section_name] = []
+                    
+                    section_to_tabs[section_name] += [ ((_(tab_name), utils.create_link(views[0]))) ]
+                    
+            if first_tab:
+                dataset["interface.sections"].append( (_(section_name), utils.create_link(first_tab)) )
+    
+ 
+        if isinstance(parameters, prewikka.view.RelativeViewParameters):
+            view_name = parameters["origin"]
+        elif view:
+            view_name = view["name"]
+        else:
+            view_name = None
+            
+        if view_name and self._view_to_section.has_key(view_name):
+            active_section = self._view_to_section[view_name]
+            active_tab = self._view_to_tab[view_name]
+            tabs = section_to_tabs[active_section]
+        
+        else:
+            active_section, tabs, active_tab = "", [ ], ""
 
+        dataset["interface.tabs"] = tabs
         dataset["prewikka.user"] = user
+        
+        if user:
+            if is_anon:
+                dataset["prewikka.userlink"] = "<b>%s</b>" % utils.escape_html_string(_(user.login))
+            else:
+                dataset["prewikka.userlink"] = "<b><a href=\"%s\">%s</a></b>" % (utils.create_link("user_settings_display"), utils.escape_html_string(user.login))
+            
+        dataset["interface.active_tab"] = _(active_tab)
+        dataset["interface.active_section"] = _(active_section)
         dataset["prewikka.logout_link"] = (user and self._env.auth.canLogout()) and utils.create_link("logout") or None
 
     def _printDataSet(self, dataset, level=0):
@@ -251,19 +275,11 @@ class Core:
     def _checkPermissions(self, request, view, user):
         if user and view.has_key("permissions"):
             if not user.has(view["permissions"]):
-                self._env.log.warning("Access to view forbidden", request, user.login)
-                raise User.PermissionDeniedError(user.login, view["name"])
+                raise User.PermissionDeniedError(view["name"])
 
     def _getParameters(self, request, view, user):
-        from prewikka.view import ParameterError
-
         parameters = view["parameters"](request.arguments) - [ "view" ]
-
-        try:
-            parameters.normalize(view["name"], user)
-        except ParameterError, e:
-            self._env.log.error("%s" % str(e), request, user.login)
-            raise InvalidQueryError(request.getQueryString())
+        parameters.normalize(view["name"], user)
 
         return parameters
         
@@ -273,48 +289,59 @@ class Core:
             return self._views[name]
 
         except KeyError:
-            self._env.log.error("View '%s' does not exist" % name, request, user.login)
-            raise InvalidQueryError(request.getQueryString())
+            raise InvalidQueryError("View '%s' does not exist" % name)
 
     def checkAuth(self, request):
-        return self._env.auth.getUser(request)
-
-    def _setupError(self, error, request, user):
-        error.dataset["query"] = request.getQueryString()
-        self._setupDataSet(error.dataset, request, user)
-
-        return error.dataset, error.template
-    
+        user = self._env.auth.getUser(request)
+        if not user.language and self._env.default_locale:
+            user.setLanguage(self._env.default_locale)
+            
+        return user
+        
     def process(self, request):
+        login = None
+        view = None
+        user = None
         try:
-            user = None
             user = self.checkAuth(request)
+            login = user.login
             view = self._getView(request, user)
 
             self._checkPermissions(request, view, user)
             parameters = self._getParameters(request, view, user)
             view_object = self._setupView(view, request, parameters, user)
-
-            self._env.log.info("Loading view", request, user.login)
+            
+            if not isinstance(view_object, Logout):
+                self._env.log.info("Loading view", request, user.login)
+            
             getattr(view_object, view["handler"])()
-
+            self._setupDataSet(view_object.dataset, request, user, view, parameters)
+            
             dataset = view_object.dataset
             template_name = view["template"]
 
             self._cleanupView(view_object)
             
-        except Error.PrewikkaError, e:
-            self._setupDataSet(e.dataset, request, user)
+        except Error.PrewikkaUserError, e:
+            if e._log_priority:
+                self._env.log.log(e._log_priority, "%s" % str(e), request=request, user=login or e._log_user)
+                
+            self._setupDataSet(e.dataset, request, user, view=view)
             dataset, template_name = e.dataset, e.template
-            
+                    
         except Exception, e:
-            error = Error.SimpleError("prewikka internal error", str(e),
-                                      display_traceback=not self._env.config.general.has_key("disable_error_traceback"))
-            init_dataset(error.dataset, self._env.config, request)
+            self._env.log.error("%s" % str(e), request, login)
+            error = Error.PrewikkaUserError("Prewikka internal error", str(e),
+                                            display_traceback=not self._env.config.general.has_key("disable_error_traceback"))
+            self._setupDataSet(error.dataset, request, user, view=view)
             dataset, template_name = error.dataset, error.template
         
         #self._printDataSet(dataset)
         template = load_template(template_name, dataset)
 
+        # We check the character set after loading the template, 
+        # since the template might trigger a language change.
+        dataset["document.charset"] = localization.getCurrentCharset()
+        
         request.content = str(template)
         request.sendResponse()
